<?php

namespace Drupal\psn_public_trophies\Component;

use PSNPublicTrophiesLib\PSNPublicTrophiesAuth;
use PSN\Auth;
use PSN\AuthException;

use PSN\User;
use PSN\Trophy;
use PSN\Friend;

/**
 * Provides methods for generating path aliases.
 */
class DrupalPSNPublicTrophies {
	public $token;
	private $config;
	private $_psn_public_trophies_auth;

	public function __construct() {
		$this->config = \Drupal::service('config.factory')->getEditable('psn_public_trophies.settings');

		$this->token = $this->config->get('psn_account_token', NULL);

		$this->_psn_public_trophies_auth = new PSNPublicTrophiesAuth($this->token);
		if ($this->token) {
			$this->config->set('psn_account_token', $this->_psn_public_trophies_auth->access_token)->save();
			$this->token = $this->_psn_public_trophies_auth->access_token;
		}
	}

	public function getAuthorizationUrl() {
		return $this->_psn_public_trophies_auth->getAuthenticateUrl();
	}

	public function connect($psn_id, $password, $ticket_uuid_url = NULL, $verification_code = NULL) {
		try {
			$auth_url_parameters = $this->_psn_public_trophies_auth->getAuthenticateUrlParameters($ticket_uuid_url);
			$ticket_uuid = NULL;
			if (isset($auth_url_parameters['ticket_uuid'])) {
				$ticket_uuid = $auth_url_parameters['ticket_uuid'];
			}

			$auth = $this->_psn_public_trophies_auth->authenticate($psn_id, $password, $ticket_uuid, $verification_code);

			$psn_account_token = $auth->GetTokens();

			$this->config->set('psn_account_token', $psn_account_token)->save();
			$this->token = $psn_account_token;

			return $psn_account_token;
		} catch (AuthException $e) {
            throw $e;
        }
	}

	public function disconnect() {
		$this->config->set('psn_account_token', NULL)->save();
		$this->token = NULL;
		return NULL;
	}

	public function refreshToken() {
		$new_token = $this->_psn_public_trophies_auth->refreshToken();
		$this->config->set('psn_account_token', $new_token)->save();
		$this->token = $new_token;

		return $new_token;
	}

	public function getUser() {
		try {
			$this->refreshToken();
			return new User($this->token);
		} catch (\Exception $e) {
			throw $e;
		}
	}

	public function getUserMe() {
		return $this->_psn_public_trophies_auth->getProfile();
	}

	public function getMyTrophies() {
		return $this->_psn_public_trophies_auth->getTrophies();
	}

	public function getFriends() {
		return $this->_psn_public_trophies_auth->getFriend();
	}

	public function getTrophy() {
		return $this->_psn_public_trophies_auth->getTrophy();
	}
}
