<?php

namespace Drupal\psn_public_trophies\Form;

use Drupal\Core\Form\ConfigFormBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Form\FormStateInterface;
use Drupal\psn_public_trophies\Component\DrupalPSNPublicTrophies;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Url;

/**
 * Edit config variable form.
 */
class PSNPublicTrophiesSettingsForm extends ConfigFormBase {
    /**
     * {@inheritdoc}
     */
    public function getFormId() {
        return 'psn_public_trophies_admin_settings_form';
    }

    /**
     * {@inheritdoc}
     */
    protected function getEditableConfigNames() {
        return [
            'psn_public_trophies.settings',
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function buildForm(array $form, FormStateInterface $form_state, Request $request = NULL) {
        $drupalPSNPublicTrophies = new DrupalPSNPublicTrophies();

        $config = $this->config('psn_public_trophies.settings');

        $form['psn_id'] = array(
            '#type' => 'textfield',
            '#title' => $this->t('PSN ID'),
            '#description' => $this->t('PSN ID from Playstation'),
            '#default_value' => $config->get('psn_id'),
        );
        $form['psn_password'] = array(
            '#type' => 'password',
            '#title' => $this->t('PSN Password'),
            '#default_value' => $config->get('psn_password'),
            '#attributes' => array('value' => $config->get('psn_password'))
        );

        $psn_account_token = $drupalPSNPublicTrophies->token;
        if (!$psn_account_token) {

            $form['psn_ticket_uuid_url'] = array(
                '#type' => 'textfield',
                '#title' => $this->t('Ticket UUID URL'),
                '#maxlength' => 3000,
                '#description' => $this->t('Click Get Ticket UUID link to login and copy the url and paste it here. DO NOT ENTER THE CODE ON THE PLAYSTATION VERIFICATION FORM.')
            );
            $form['psn_get_ticket_uuid'] = array(
                '#type' => 'link',
                '#title' => $this->t('Get Ticket UUID'),
                '#url' => Url::fromUri($drupalPSNPublicTrophies->getAuthorizationUrl()),
                '#attributes' => array('target' => '_blank')
            );
            $form['psn_verification_code'] = array(
                '#type' => 'textfield',
                '#title' => $this->t('Verification Code'),
                '#description' => $this->t('Enter the verification code sent on your mobile device here. DO NOT ENTER THE CODE ON THE PLAYSTATION VERIFICATION FORM.')
            );

            $form['psn_connect'] = array(
                '#type' => 'submit',
                '#value' => $this->t('Connect'),
                '#submit' => array('::psnConnectSubmit'),
            );
        } else {
            $form['psn_account_token'] = array(
                '#type' => 'textarea',
                '#title' => $this->t('Access Token'),
                '#default_value' => Json::encode($psn_account_token)
            );

            $form['psn_refresh_token'] = array(
                    '#type' => 'submit',
                    '#value' => $this->t('Refresh Token'),
                    '#submit' => array('::psnRefreshTokenSubmit'),
            );

            $form['psn_disconnect'] = array(
                    '#type' => 'submit',
                    '#value' => $this->t('Disconnect'),
                    '#submit' => array('::psnDisconnectSubmit'),
            );

            $form['psn_get_user'] = array(
                '#type' => 'submit',
                '#value' => $this->t('Get Data'),
                '#submit' => array('::psnGetUserSubmit'),
            );
        }

        return parent::buildForm($form, $form_state);
    }

    public function psnDisconnectSubmit(array &$form, FormStateInterface $form_state) {
        $drupalPSNPublicTrophies = new DrupalPSNPublicTrophies();
        $drupalPSNPublicTrophies->disconnect();
    }

    public function psnGetUserSubmit(array &$form, FormStateInterface $form_state) {
        $drupalPSNPublicTrophies = new DrupalPSNPublicTrophies();
        try {
            $trophy = $drupalPSNPublicTrophies->getTrophy();
            $user = $drupalPSNPublicTrophies->getUser();
            $friends = $drupalPSNPublicTrophies->getFriends();

            $messenger = \Drupal::messenger();
            $messenger->addMessage('Me: ' . Json::encode($drupalPSNPublicTrophies->getUserMe()));
            $messenger->addMessage('My Friends: ' . Json::encode($friends->MyFriends()));
            $messenger->addMessage('My Trophies: ' . Json::encode($drupalPSNPublicTrophies->getMyTrophies()));
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function psnRefreshTokenSubmit(array &$form, FormStateInterface $form_state) {
        $drupalPSNPublicTrophies = new DrupalPSNPublicTrophies();
        $drupalPSNPublicTrophies->refreshToken();
    }

    public function psnConnectSubmit(array &$form, FormStateInterface $form_state) {
        $drupalPSNPublicTrophies = new DrupalPSNPublicTrophies();
        $config = $this->config('psn_public_trophies.settings');

        if ($psn_id = $config->get('psn_id') && $psn_password = $config->get('psn_password')) {
            try {
                $psn_account_token = $drupalPSNPublicTrophies->connect($psn_id, $psn_password, $form_state->getValue('psn_ticket_uuid_url'), $form_state->getValue('psn_verification_code'));
            } catch (\PSN\AuthException $e) {
                header("Content-Type: application/json");
                die($e->GetError());
            }
        }
    }

    /**
     * {@inheritdoc}
     */
    public function submitForm(array &$form, FormStateInterface $form_state) {
        $values = $form_state->getValues();
        $this->config('psn_public_trophies.settings')
            ->set('psn_id', $values['psn_id']);
        if ($values['psn_password']) {
            $this->config('psn_public_trophies.settings')
                ->set('psn_password', $values['psn_password']);
        }

        $drupalPSNPublicTrophies = new DrupalPSNPublicTrophies();
        $drupalPSNPublicTrophies->disconnect();

        parent::submitForm($form, $form_state);
    }
}
